@@ -3,6 +3,7 @@
 #include <QtEndian>
 #include "logger.hh"
 #include "roamingchannel.hh"
+#include "configcopyvisitor.hh"
 
 
 /* ********************************************************************************************* *
@@ -34,6 +35,13 @@ Codeplug::Element::~Element() {
   // pass...
 }
 
+Codeplug::Element &
+Codeplug::Element::operator =(const Element &other) {
+  this->_data = other._data;
+  this->_size = other._size;
+  return *this;
+}
+
 bool
 Codeplug::Element::isValid() const {
   return nullptr != _data;
@@ -57,6 +65,11 @@ Codeplug::Element::fill(uint8_t value, unsigned offset, int size) {
   return true;
 }
 
+bool
+Codeplug::Element::getBit(const Offset::BitOffset &offset) const {
+  return getBit(offset.byte, offset.bit);
+}
+
 bool
 Codeplug::Element::getBit(unsigned offset, unsigned bit) const {
   if (offset >= _size) {
@@ -69,6 +82,10 @@ Codeplug::Element::getBit(unsigned offset, unsigned bit) const {
   return (1<<bit) & (*ptr);
 }
 
+void
+Codeplug::Element::setBit(const Offset::BitOffset &offset, bool value) {
+  setBit(offset.byte, offset.bit, value);
+}
 void
 Codeplug::Element::setBit(unsigned offset, unsigned bit, bool value) {
   if (offset >= _size) {
@@ -598,6 +615,7 @@ Codeplug::Context::Context(Config *config)
   addTable(&APRSSystem::staticMetaObject);
   addTable(&RoamingChannel::staticMetaObject);
   addTable(&RoamingZone::staticMetaObject);
+  addTable(&SMSTemplate::staticMetaObject);
 }
 
 Config  *
@@ -650,12 +668,10 @@ bool
 Codeplug::Context::add(ConfigItem *obj, unsigned idx) {
   if (!hasTable(obj->metaObject()))
     return false;
-  if (getTable(obj->metaObject()).indices.contains(obj))
-    return false;
-  if (getTable(obj->metaObject()).objects.contains(idx))
-    return false;
-  getTable(obj->metaObject()).objects.insert(idx, obj);
-  getTable(obj->metaObject()).indices.insert(obj, idx);
+  if (! getTable(obj->metaObject()).indices.contains(obj))
+    getTable(obj->metaObject()).indices.insert(obj, idx);
+  if (! getTable(obj->metaObject()).objects.contains(idx))
+    getTable(obj->metaObject()).objects.insert(idx, obj);
   return true;
 }
 
@@ -672,3 +688,14 @@ Codeplug::Codeplug(QObject *parent)
 Codeplug::~Codeplug() {
 	// pass...
 }
+
+Config *
+Codeplug::preprocess(Config *config, const ErrorStack &err) const {
+  return ConfigCopy::copy(config, err)->as<Config>();
+}
+
+bool
+Codeplug::postprocess(Config *config, const ErrorStack &err) const {
+  Q_UNUSED(config); Q_UNUSED(err);
+  return true;
+}
