@@ -3,7 +3,6 @@
 
 #include "rxgrouplist.hh"
 #include "channel.hh"
-#include "encryptionextension.hh"
 #include "csvreader.hh"
 #include "userdatabase.hh"
 #include "logger.hh"
@@ -25,7 +24,8 @@ Config::Config(QObject *parent)
     _zones(new ZoneList(this)), _scanlists(new ScanLists(this)),
     _gpsSystems(new PositioningSystems(this)),
     _roamingChannels(new RoamingChannelList(this)), _roamingZones(new RoamingZoneList(this)),
-    _tytExtension(nullptr), _commercialExtension(new CommercialExtension(this))
+    _tytExtension(nullptr), _commercialExtension(new CommercialExtension(this)),
+    _smsExtension(new SMSExtension(this))
 {
   connect(_settings, SIGNAL(modified(ConfigItem*)), this, SLOT(onConfigModified()));
   connect(_radioIDs, SIGNAL(elementAdded(int)), this, SLOT(onConfigModified()));
@@ -57,6 +57,7 @@ Config::Config(QObject *parent)
   connect(_roamingZones, SIGNAL(elementModified(int)), this, SLOT(onConfigModified()));
 
   connect(_commercialExtension, SIGNAL(modified(ConfigItem*)), this, SLOT(onConfigModified()));
+  connect(_smsExtension, SIGNAL(modified(ConfigItem*)), this, SLOT(onConfigModified()));
 }
 
 bool
@@ -111,7 +112,7 @@ Config::toYAML(QTextStream &stream, const ErrorStack &err) {
   // Print YAML
   YAML::Emitter emitter;
   emitter << YAML::BeginDoc << doc << YAML::EndDoc;
-  stream << emitter.c_str();
+  stream << QString(emitter.c_str());
   return true;
 }
 
@@ -123,9 +124,6 @@ Config::populate(YAML::Node &node, const Context &context, const ErrorStack &err
   if ((node["settings"]= _settings->serialize(context, err)).IsNull())
     return false;
 
-  if (_radioIDs->defaultId() && context.contains(_radioIDs->defaultId()))
-    node["settings"]["defaultID"] = context.getId(_radioIDs->defaultId()).toStdString();
-
   if ((node["radioIDs"] = _radioIDs->serialize(context, err)).IsNull())
     return false;
 
@@ -280,6 +278,11 @@ Config::commercialExtension() const {
   return _commercialExtension;
 }
 
+SMSExtension *
+Config::smsExtension() const {
+  return _smsExtension;
+}
+
 TyTConfigExtension *
 Config::tytExtension() const {
   return _tytExtension;
@@ -395,7 +398,7 @@ Config::parse(const YAML::Node &node, Context &ctx, const ErrorStack &err)
     return false;
   if (node["roamingZones"] && (! _roamingZones->parse(node["roamingZones"], ctx, err)))
     return false;
-  /** @todo Implemented for backward compatability with version 0.10.0, remove for 1.0.0.*/
+  /** @todo Implemented for backward compatibility with version 0.10.0, remove for 1.0.0.*/
   else if (node["roaming"] && (! _roamingZones->parse(node["roaming"], ctx, err)))
     return false;
 
@@ -410,29 +413,12 @@ bool
 Config::link(const YAML::Node &node, const Context &ctx, const ErrorStack &err) {
   // radio IDs must be linked before settings, as they may refer to the default DMR ID
 
-  if (! _radioIDs->link(node["radioIDs"], ctx, err))
+  if (node["radioIDs"] && (! _radioIDs->link(node["radioIDs"], ctx, err)))
     return false;
 
-  if (! _settings->link(node["settings"], ctx, err))
-    return false;
-
-  // Link default radio ID separately as it is not a property of the settings but defined there
-  if (node["settings"] && node["settings"]["defaultID"] && node["settings"]["defaultID"].IsScalar()) {
-    YAML::Node defIDNode = node["settings"]["defaultID"];
-    QString id = QString::fromStdString(defIDNode.as<std::string>());
-    if (ctx.contains(id) && ctx.getObj(id)->is<DMRRadioID>()) {
-      DMRRadioID *def = ctx.getObj(id)->as<DMRRadioID>();
-      radioIDs()->setDefaultId(radioIDs()->indexOf(def));
-      logDebug() << "Set default radio ID to '" << def->name() << "'.";
-    } else {
-      errMsg(err) << defIDNode.Mark().line << ":" << defIDNode.Mark().column
-                  << "Default radio ID '" << id << " does not refer to a radio ID.";
+  if (node["settings"])
+    if (!_settings->link(node["settings"], ctx, err))
       return false;
-    }
-  } else if (radioIDs()->count()) {
-    // If no default is set, use first one.
-    radioIDs()->setDefaultId(0);
-  }
 
   if (node["contacts"] && (! _contacts->link(node["contacts"], ctx, err)))
     return false;
@@ -448,7 +434,7 @@ Config::link(const YAML::Node &node, const Context &ctx, const ErrorStack &err)
     return false;
   if (node["roamingZones"] && (! _roamingZones->link(node["roamingZones"], ctx, err)))
     return false;
-  /** @todo Implemented for backward compatability with version 0.10.0, remove for 1.0.0.*/
+  /** @todo Implemented for backward compatibility with version 0.10.0, remove for 1.0.0.*/
   else if (node["roaming"] && (! _roamingZones->link(node["roaming"], ctx, err)))
     return false;
 
