@@ -1,6 +1,9 @@
 #include "configobject.hh"
 #include "configreference.hh"
 #include "logger.hh"
+#include "frequency.hh"
+#include "interval.hh"
+#include "commercial_extension.hh"
 
 #include <QMetaProperty>
 #include <QMetaEnum>
@@ -161,7 +164,9 @@ ConfigItem::copy(const ConfigItem &other) {
     // true if the property is a basic type
     bool isBasicType = ( prop.isEnumType() || (QVariant::Bool==prop.type()) ||
                          (QVariant::Int==prop.type()) || (QVariant::UInt==prop.type()) ||
-                         (QVariant::Double==prop.type()) ||(QVariant::String==prop.type()));
+                         (QVariant::Double==prop.type()) || (QVariant::String==prop.type()) ||
+                         (QString("Frequency")==prop.typeName()) ||
+                         (QString("Interval")==prop.typeName()) );
 
     // If a basic type -> simply copy value
     if (isBasicType && prop.isWritable() && (prop.type()==oprop.type())) {
@@ -250,52 +255,80 @@ ConfigItem::compare(const ConfigItem &other) const {
     // Handle comparison of basic types
     if ((prop.isEnumType()) || (QVariant::Bool == prop.type()) || (QVariant::Int == prop.type()) || (QVariant::UInt == prop.type())) {
       int a=prop.read(this).toInt(), b=oprop.read(&other).toInt();
-      if (a<b) return -1;
-      if (a>b) return 1;
+      if (a<b)
+        return -1;
+      if (a>b)
+        return 1;
       continue;
     }
 
     if (QVariant::Double == prop.type()) {
       double a=prop.read(this).toDouble(), b=oprop.read(&other).toDouble();
-      if (a<b) return -1;
-      if (a>b) return 1;
+      if (a<b)
+        return -1;
+      if (a>b)
+        return 1;
       continue;
     }
 
     if (QVariant::String == prop.type()) {
       int cmp = QString::compare(prop.read(this).toString(), oprop.read(&other).toString());
-      if (cmp) return cmp;
+      if (cmp)
+        return cmp;
+      continue;
+    }
+
+    if (QString("Frequency") == prop.typeName()) {
+      Frequency a = prop.read(this).value<Frequency>(), b = oprop.read(&other).value<Frequency>();
+      if (a<b)
+        return -1;
+      if (b<a)
+        return 1;
+      continue;
+    }
+
+    if (QString("Interval") == prop.typeName()) {
+      Interval a = prop.read(this).value<Interval>(), b = oprop.read(&other).value<Interval>();
+      if (a<b)
+        return -1;
+      if (b<a)
+        return 1;
       continue;
     }
 
     if (ConfigObjectReference *ref = prop.read(this).value<ConfigObjectReference *>()) {
       int cmp = ref->compare(*oprop.read(&other).value<ConfigObjectReference*>());
-      if (cmp) return cmp;
+      if (cmp)
+        return cmp;
       continue;
     }
 
     if (ConfigObjectList *lst = prop.read(this).value<ConfigObjectList *>()) {
       int cmp = lst->compare(*oprop.read(&other).value<ConfigObjectList*>());
-      if (cmp) return cmp;
+      if (cmp)
+        return cmp;
       continue;
     }
 
-    if (ConfigObjectRefList *lst = prop.read(this).value<ConfigObjectRefList *>()) {
+    if (propIsInstance<ConfigObjectRefList>(prop)) {
+      ConfigObjectRefList *lst = prop.read(this).value<ConfigObjectRefList *>();
       int cmp = lst->compare(*oprop.read(&other).value<ConfigObjectRefList*>());
-      if (cmp) return cmp;
+      if (cmp)
+        return cmp;
       continue;
     }
 
     if (propIsInstance<ConfigItem>(prop)) {
       // If the owned item is writeable -> clone if set in other
-      if (prop.read(&other).isNull() && !oprop.read(&other).isNull())
+      if (prop.read(this).isNull() && !oprop.read(&other).isNull())
         return -1;
-      if (!prop.read(&other).isNull() && oprop.read(&other).isNull())
+      if (!prop.read(this).isNull() && oprop.read(&other).isNull())
         return 1;
-      if (prop.read(&other).isNull() && oprop.read(&other).isNull())
+      if (prop.read(this).isNull() && oprop.read(&other).isNull())
         continue;
-      int cmp = prop.read(&other).value<ConfigItem*>()->compare(*oprop.read(&other).value<ConfigItem*>());
-      if (cmp) return cmp;
+      int cmp = prop.read(this).value<ConfigItem*>()->compare(*oprop.read(&other).value<ConfigItem*>());
+      if (cmp)
+        return cmp;
       continue;
     }
   }
@@ -378,7 +411,8 @@ ConfigItem::populate(YAML::Node &node, const Context &context, const ErrorStack
       if (nullptr == key) {
         errMsg(err) << "Cannot map value " << value.toUInt()
                     << " to enum " << e.name()
-                    << ". Ignore attribute but this points to an incompatibility in some codeplug. "
+                    << ". Ignore attribute '" << prop.name()
+                    << "' but this points to an incompatibility in some codeplug. "
                     << "Consider reporting it to https://github.com/hmatuschek/qdmr/issues.";
         continue;
       }
@@ -393,6 +427,10 @@ ConfigItem::populate(YAML::Node &node, const Context &context, const ErrorStack
       node[prop.name()] = this->property(prop.name()).toDouble();
     } else if (QString("QString") == prop.typeName()) {
       node[prop.name()] = this->property(prop.name()).toString().toStdString();
+    } else if (QString("Frequency") == prop.typeName()) {
+      node[prop.name()] = this->property(prop.name()).value<Frequency>();
+    } else if (QString("Interval") == prop.typeName()) {
+      node[prop.name()] = this->property(prop.name()).value<Interval>();
     } else if (ConfigObjectReference *ref = prop.read(this).value<ConfigObjectReference *>()) {
       ConfigObject *obj = ref->as<ConfigObject>();
       if (nullptr == obj)
@@ -582,6 +620,31 @@ ConfigItem::parse(const YAML::Node &node, ConfigItem::Context &ctx, const ErrorS
         return false;
       }
       prop.write(this, QString::fromStdString(node[prop.name()].as<std::string>()));
+    } else if (QString("Frequency") == prop.typeName()) {
+      // If property is not set -> skip
+      if (! node[prop.name()])
+        continue;
+      // parse & check type
+      if (! node[prop.name()].IsScalar()) {
+        errMsg(err) << node[prop.name()].Mark().line << ":" << node[prop.name()].Mark().column
+                    << ": Cannot parse " << prop.name() << " of " << meta->className()
+                    << ": Expected frequency.";
+        return false;
+      }
+      Frequency f = node[prop.name()].as<Frequency>();
+      prop.write(this, QVariant::fromValue(f));
+    } else if (QString("Interval") == prop.typeName()) {
+      // If property is not set -> skip
+      if (! node[prop.name()])
+        continue;
+      // parse & check type
+      if (! node[prop.name()].IsScalar()) {
+        errMsg(err) << node[prop.name()].Mark().line << ":" << node[prop.name()].Mark().column
+                    << ": Cannot parse " << prop.name() << " of " << meta->className()
+                    << ": Expected interval.";
+        return false;
+      }
+      prop.write(this, QVariant::fromValue(node[prop.name()].as<Interval>()));
     } else if (prop.read(this).value<ConfigObjectReference *>()) {
       // references are linked later
       continue;
@@ -1095,16 +1158,33 @@ AbstractConfigObjectList::findItemsOfTypes(const QStringList &typeNames, QSet<Co
   }
 }
 
-ConfigObject *AbstractConfigObjectList::get(int idx) const {
+QList<ConfigObject *>
+AbstractConfigObjectList::findItemsByName(const QString name) const {
+  QList<ConfigObject *> items;
+  foreach (ConfigObject *obj, _items) {
+    if (obj->name() == name)
+      items.append(obj);
+  }
+  return items;
+}
+
+bool
+AbstractConfigObjectList::has(ConfigObject *obj) const {
+  return 0 <= indexOf(obj);
+}
+
+ConfigObject *
+AbstractConfigObjectList::get(int idx) const {
   return _items.value(idx, nullptr);
 }
 
-int AbstractConfigObjectList::add(ConfigObject *obj, int row) {
+int
+AbstractConfigObjectList::add(ConfigObject *obj, int row, bool unique) {
   // Ignore nullptr
   if (nullptr == obj)
     return -1;
   // If already in list -> ignore
-  if (0 <= indexOf(obj))
+  if (unique && (0 <= indexOf(obj)))
     return -1;
   if (-1 == row)
     row = _items.size();
@@ -1129,6 +1209,49 @@ int AbstractConfigObjectList::add(ConfigObject *obj, int row) {
   return row;
 }
 
+int
+AbstractConfigObjectList::replace(ConfigObject *obj, int row, bool unique) {
+  // Ignore nullptr
+  if (nullptr == obj)
+    return -1;
+  // Check index
+  if (row >= count())
+    return -1;
+  // Check if self-replacement
+  if (row == indexOf(obj))
+    return indexOf(obj);
+  // If already in list -> ignore
+  if (unique && (0 <= indexOf(obj)))
+    return -1;
+  // Check type
+  bool matchesType = false;
+  foreach (const QMetaObject &type, _elementTypes) {
+    if (obj->inherits(type.className())) {
+      matchesType = true;
+      break;
+    }
+  }
+  if (! matchesType) {
+    logError() << "Cannot add element of type " << obj->metaObject()->className()
+               << " to list, expected instances of " << classNames().join(", ");
+    return -1;
+  }
+
+  // Remove present element
+  ConfigObject *oldobj = _items.at(row);
+  _items.remove(row, 1);
+  emit elementRemoved(row);
+  disconnect(oldobj, nullptr, this, nullptr);
+
+  _items.insert(row, obj);
+  // connect to object
+  connect(obj, SIGNAL(destroyed(QObject*)), this, SLOT(onElementDeleted(QObject*)));
+  connect(obj, SIGNAL(modified(ConfigItem*)), this, SLOT(onElementModified(ConfigItem*)));
+  emit elementAdded(row);
+
+  return row;
+}
+
 bool
 AbstractConfigObjectList::take(ConfigObject *obj) {
   // Ignore nullptr
@@ -1183,6 +1306,24 @@ AbstractConfigObjectList::moveDown(int first, int last) {
   return true;
 }
 
+bool
+AbstractConfigObjectList::move(int source, int count, int destination) {
+  if ((0 == count) || (source == destination))
+    return true;
+  if ((source+count)>_items.size())
+    return false;
+  if (source > destination) {
+    // move up
+    for (int take=source, put=destination, i=0; i<count; i++, take++, put++)
+      _items.insert(put, _items.takeAt(take));
+  } else {
+    // move down
+    for (int i=0; i<count; i++)
+      _items.insert(destination-1, _items.takeAt(source));
+  }
+  return true;
+}
+
 const QList<QMetaObject> &
 AbstractConfigObjectList::elementTypes() const {
   return _elementTypes;
@@ -1316,8 +1457,8 @@ ConfigObjectList::link(const YAML::Node &node, const ConfigItem::Context &ctx, c
   return true;
 }
 
-int ConfigObjectList::add(ConfigObject *obj, int row) {
-  if (0 <= (row = AbstractConfigObjectList::add(obj, row)))
+int ConfigObjectList::add(ConfigObject *obj, int row, bool unique) {
+  if (0 <= (row = AbstractConfigObjectList::add(obj, row, unique)))
     obj->setParent(this);
   return row;
 }
@@ -1332,7 +1473,7 @@ ConfigObjectList::take(ConfigObject *obj) {
 bool
 ConfigObjectList::del(ConfigObject *obj) {
   if (AbstractConfigObjectList::del(obj))
-    obj->deleteLater();
+    delete obj;
   return true;
 }
 
