@@ -1766,8 +1766,8 @@ CSVReader::handleRadioName(const QString &name, qint64 line, qint64 column, QStr
   Q_UNUSED(column);
   Q_UNUSED(errorMessage);
 
-  if (_link && _config->radioIDs()->defaultId()) {
-    _config->radioIDs()->defaultId()->setName(name);
+  if (_link && (!_config->settings()->defaultIdRef()->isNull())) {
+    _config->settings()->defaultIdRef()->as<DMRRadioID>()->setName(name);
   }
   return true;
 }
@@ -1981,8 +1981,8 @@ CSVReader::handleDigitalChannel(qint64 idx, const QString &name, double rx, doub
 
   DMRChannel *chan = new DMRChannel();
   chan->setName(name);
-  chan->setRXFrequency(rx);
-  chan->setTXFrequency(tx);
+  chan->setRXFrequency(Frequency::fromMHz(rx));
+  chan->setTXFrequency(Frequency::fromMHz(tx));
   chan->setPower(power);
   chan->setTimeout(tot);
   chan->setRXOnly(ro);
@@ -2039,8 +2039,8 @@ CSVReader::handleAnalogChannel(qint64 idx, const QString &name, double rx, doubl
 
   FMChannel *chan = new FMChannel();
   chan->setName(name);
-  chan->setRXFrequency(rx);
-  chan->setTXFrequency(tx);
+  chan->setRXFrequency(Frequency::fromMHz(rx));
+  chan->setTXFrequency(Frequency::fromMHz(tx));
   chan->setPower(power);
   chan->setTimeout(tot);
   chan->setRXOnly(ro);
