@@ -6,7 +6,6 @@
 #include <QDateTime>
 
 #define ADDR_SETTINGS             0x0000e0
-#define ADDR_BUTTONS              0x000108
 #define ADDR_MESSAGE_BANK         0x000128
 
 #define ADDR_ENCRYPTION           0x001370
@@ -42,9 +41,8 @@
 #define ADDR_CONTACTS             0x017620
 #define CONTACT_SIZE              0x000018
 
-#define NUM_GROUP_LISTS                 76
+#define NUM_GROUP_LISTS                 64
 #define ADDR_GROUP_LIST_BANK      0x01d620
-#define GROUPLIST_SIZE            0x000050
 #define GROUP_LIST_BANK_SIZE      0x001840
 
 
@@ -223,7 +221,7 @@ GD77Codeplug::GroupListElement::GroupListElement(uint8_t *ptr, unsigned size)
 }
 
 GD77Codeplug::GroupListElement::GroupListElement(uint8_t *ptr)
-  : RadioddityCodeplug::GroupListElement(ptr, 0x0050)
+  : RadioddityCodeplug::GroupListElement(ptr, size())
 {
   // pass...
 }
@@ -239,14 +237,14 @@ GD77Codeplug::GroupListBankElement::GroupListBankElement(uint8_t *ptr, unsigned
 }
 
 GD77Codeplug::GroupListBankElement::GroupListBankElement(uint8_t *ptr)
-  : RadioddityCodeplug::GroupListBankElement(ptr, 0x1840)
+  : RadioddityCodeplug::GroupListBankElement(ptr, size())
 {
   // pass...
 }
 
 uint8_t *
 GD77Codeplug::GroupListBankElement::get(unsigned n) const {
-  return _data + 0x80 + n*GROUPLIST_SIZE;
+  return _data + Offset::groupLists() + n*GroupListElement::size();
 }
 
 
@@ -258,7 +256,7 @@ GD77Codeplug::GD77Codeplug(QObject *parent)
 {
   addImage("Radioddity GD77 Codeplug");
   image(0).addElement(0x00080, 0x07b80);
-  image(0).addElement(0x08000, 0x16300);
+  image(0).addElement(0x08000, 0x16b00);
 }
 
 void
@@ -284,13 +282,30 @@ GD77Codeplug::decodeGeneralSettings(Config *config, Context &ctx, const ErrorSta
 
 void
 GD77Codeplug::clearButtonSettings() {
-  ButtonSettingsElement(data(ADDR_BUTTONS)).clear();
+  ButtonSettingsElement(data(Offset::buttonSettings())).clear();
+}
+bool
+GD77Codeplug::encodeButtonSettings(Context &ctx, const Flags &flags, const ErrorStack &err) {
+  Q_UNUSED(flags);
+  return ButtonSettingsElement(data(Offset::buttonSettings())).encode(ctx, err);
+}
+bool
+GD77Codeplug::decodeButtonSettings(Context &ctx, const ErrorStack &err) {
+  return ButtonSettingsElement(data(Offset::buttonSettings())).decode(ctx, err);
 }
 
 void
 GD77Codeplug::clearMessages() {
   MessageBankElement(data(ADDR_MESSAGE_BANK)).clear();
 }
+bool
+GD77Codeplug::encodeMessages(Context &ctx, const Flags &flags, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).encode(ctx, flags, err);
+}
+bool
+GD77Codeplug::decodeMessages(Context &ctx, const ErrorStack &err) {
+  return MessageBankElement(data(Offset::messages())).decode(ctx, err);
+}
 
 void
 GD77Codeplug::clearScanLists() {
@@ -465,36 +480,21 @@ GD77Codeplug::clearZones() {
 
 bool
 GD77Codeplug::encodeZones(Config *config, const Flags &flags, Context &ctx, const ErrorStack &err) {
-  Q_UNUSED(flags); Q_UNUSED(err)
+  Q_UNUSED(flags); Q_UNUSED(err); Q_UNUSED(config)
 
   ZoneBankElement bank(data(ADDR_ZONE_BANK));
 
   // Pack Zones
-  bool pack_zone_a = true;
-  for (int i=0, j=0; i<NUM_ZONES; i++) {
+  for (int i=0; i<NUM_ZONES; i++) {
     ZoneElement z(bank.get(i));
-next:
-    if (j >= config->zones()->count()) {
+    if (! ctx.has<Zone>(i+1)) {
       bank.enable(i, false);
       continue;
     }
 
     // Construct from Zone obj
-    Zone *zone = config->zones()->zone(j);
-    if (pack_zone_a) {
-      pack_zone_a = false;
-      if (zone->A()->count())
-        z.fromZoneObjA(zone, ctx);
-      else
-        goto next;
-    } else {
-      pack_zone_a = true;
-      j++;
-      if (zone->B()->count())
-        z.fromZoneObjB(zone, ctx);
-      else
-        goto next;
-    }
+    Zone *zone = ctx.get<Zone>(i+1);
+    z.fromZoneObjA(zone, ctx);
     bank.enable(i, true);
   }
   return true;
@@ -504,33 +504,14 @@ bool
 GD77Codeplug::createZones(Config *config, Context &ctx, const ErrorStack &err) {
   Q_UNUSED(err)
 
-  QString last_zonename, last_zonebasename; Zone *last_zone = nullptr;
-  bool extend_last_zone = false;
   ZoneBankElement bank(data(ADDR_ZONE_BANK));
-
   for (int i=0; i<NUM_ZONES; i++) {
     if (! bank.isEnabled(i))
       continue;
     ZoneElement z(bank.get(i));
-
-    // Determine whether this zone should be combined with the previous one
-    QString zonename = z.name();
-    QString zonebasename = zonename; zonebasename.chop(2);
-    extend_last_zone = ( zonename.endsWith(" B") && last_zonename.endsWith(" A")
-                         && (zonebasename == last_zonebasename)
-                         && (nullptr != last_zone) && (0 == last_zone->B()->count()) );
-    last_zonename = zonename;
-    last_zonebasename = zonebasename;
-
-    // Create zone obj
-    if (! extend_last_zone) {
-      last_zone = z.toZoneObj(ctx);
-      config->zones()->add(last_zone);
-      ctx.add(last_zone, i+1);
-    } else {
-      // when extending the last zone, chop its name to remove the "... A" part.
-      last_zone->setName(last_zonebasename);
-    }
+    Zone *zone = z.toZoneObj(ctx);
+    config->zones()->add(zone);
+    ctx.add(zone, i+1);
   }
   return true;
 }
@@ -539,29 +520,13 @@ bool
 GD77Codeplug::linkZones(Config *config, Context &ctx, const ErrorStack &err) {
   Q_UNUSED(config);
 
-  QString last_zonename, last_zonebasename; Zone *last_zone = nullptr;
-  bool extend_last_zone = false;
   ZoneBankElement bank(data(ADDR_ZONE_BANK));
-
   for (int i=0; i<NUM_ZONES; i++) {
     if (! bank.isEnabled(i))
       continue;
     ZoneElement z(bank.get(i));
-
-    // Determine whether this zone should be combined with the previous one
-    QString zonename = z.name();
-    QString zonebasename = zonename; zonebasename.chop(2);
-    extend_last_zone = ( zonename.endsWith(" B") && last_zonename.endsWith(" A")
-                         && (zonebasename == last_zonebasename)
-                         && (nullptr != last_zone) && (0 == last_zone->B()->count()) );
-    last_zonename = zonename;
-    last_zonebasename = zonebasename;
-
-    // Create zone obj
-    if (! extend_last_zone) {
-      last_zone = ctx.get<Zone>(i+1);
-    }
-    if (! z.linkZoneObj(last_zone, ctx, extend_last_zone)) {
+    Zone *zone = ctx.get<Zone>(i+1);
+    if (! z.linkZoneObj(zone, ctx)) {
       errMsg(err) << "Cannot link zone at index " << i << ".";
       return false;
     }
@@ -650,14 +615,14 @@ GD77Codeplug::clearGroupLists() {
 
 bool
 GD77Codeplug::encodeGroupLists(Config *config, const Flags &flags, Context &ctx, const ErrorStack &err) {
-  Q_UNUSED(flags); Q_UNUSED(err)
+  Q_UNUSED(flags)
 
   GroupListBankElement bank(data(ADDR_GROUP_LIST_BANK)); bank.clear();
   for (int i=0; i<NUM_GROUP_LISTS; i++) {
     if (i >= config->rxGroupLists()->count())
       continue;
     GroupListElement el(bank.get(i));
-    el.fromRXGroupListObj(config->rxGroupLists()->list(i), ctx);
+    el.fromRXGroupListObj(config->rxGroupLists()->list(i), ctx, err);
     bank.setContactCount(i, config->rxGroupLists()->list(i)->count());
   }
   return true;
@@ -665,14 +630,12 @@ GD77Codeplug::encodeGroupLists(Config *config, const Flags &flags, Context &ctx,
 
 bool
 GD77Codeplug::createGroupLists(Config *config, Context &ctx, const ErrorStack &err) {
-  Q_UNUSED(err)
-
   GroupListBankElement bank(data(ADDR_GROUP_LIST_BANK));
   for (int i=0; i<NUM_GROUP_LISTS; i++) {
     if (! bank.isEnabled(i))
       continue;
     GroupListElement el(bank.get(i));
-    RXGroupList *list = el.toRXGroupListObj(ctx);
+    RXGroupList *list = el.toRXGroupListObj(ctx, err);
     config->rxGroupLists()->add(list); ctx.add(list, i+1);
   }
   return true;
@@ -689,7 +652,7 @@ GD77Codeplug::linkGroupLists(Config *config, Context &ctx, const ErrorStack &err
     GroupListElement el(bank.get(i));
     /*logDebug() << "Link " << bank.contactCount(i) << " members of group list '"
                << ctx.get<RXGroupList>(i+1)->name() << "'.";*/
-    if (! el.linkRXGroupListObj(bank.contactCount(i), ctx.get<RXGroupList>(i+1), ctx)) {
+    if (! el.linkRXGroupListObj(bank.contactCount(i), ctx.get<RXGroupList>(i+1), ctx, err)) {
       errMsg(err) << "Cannot link group list '" << ctx.get<RXGroupList>(i+1)->name() << "'.";
       return false;
     }
