@@ -31,6 +31,14 @@ OpenGD77Interface::ReadRequest::initReadFlash(uint32_t addr, uint16_t length) {
   return true;
 }
 
+bool
+OpenGD77Interface::ReadRequest::initReadFirmwareInfo() {
+  this->type = 'R';
+  this->command = READ_FIRMWARE_INFO;
+  this->address = 0;
+  this->length = 0;
+  return true;
+}
 
 /* ********************************************************************************************* *
  * Implementation of OpenGD77Interface::WriteRequest
@@ -160,7 +168,7 @@ OpenGD77Interface::CommandRequest::initCommand(Option option) {
  * Implementation of OpenGD77Interface
  * ********************************************************************************************* */
 OpenGD77Interface::OpenGD77Interface(const USBDeviceDescriptor &descr, const ErrorStack &err, QObject *parent)
-  : USBSerial(descr, err, parent), _sector(-1)
+  : USBSerial(descr, QSerialPort::Baud115200, err, parent), _sector(-1)
 {
   // pass...
 }
@@ -175,8 +183,9 @@ OpenGD77Interface::interfaceInfo() {
 }
 
 QList<USBDeviceDescriptor>
-OpenGD77Interface::detect() {
-  return USBSerial::detect(USB_VID, USB_PID);
+OpenGD77Interface::detect(bool saveOnly) {
+  Q_UNUSED(saveOnly)
+  return USBSerial::detect(USB_VID, USB_PID, true);
 }
 
 void
@@ -618,7 +627,7 @@ OpenGD77Interface::sendShowCPSScreen(const ErrorStack &err) {
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
@@ -652,7 +661,7 @@ OpenGD77Interface::sendClearScreen(const ErrorStack &err) {
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
@@ -684,7 +693,7 @@ OpenGD77Interface::sendDisplay(uint8_t x, uint8_t y, const char *message, uint8_
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
@@ -716,7 +725,7 @@ OpenGD77Interface::sendRenderCPS(const ErrorStack &err) {
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
@@ -747,7 +756,7 @@ OpenGD77Interface::sendCloseScreen(const ErrorStack &err) {
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
@@ -778,7 +787,7 @@ OpenGD77Interface::sendCommand(CommandRequest::Option option, const ErrorStack &
     errMsg(err) << "Cannot send command: Device returned empty message.";
     return false;
   } else if ('-' != resp) {
-    errMsg(err) << "Cannot send command: Deviced returned unexpected response '" << (char)resp << "'.";
+    errMsg(err) << "Cannot send command: Device returned unexpected response '" << (char)resp << "'.";
     return false;
   }
 
