@@ -1,6 +1,7 @@
 #include "configitemwrapper.hh"
 #include <cmath>
 #include "logger.hh"
+#include "utils.hh"
 #include <QColor>
 #include <QPalette>
 #include <QWidget>
@@ -37,48 +38,30 @@ GenericListWrapper::columnCount(const QModelIndex &index) const {
   return 1;
 }
 
-bool
-GenericListWrapper::moveUp(int row) {
-  if ((0>=row) || (row>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), row, row, QModelIndex(), row-1);
-  _list->moveUp(row);
-  endMoveRows();
-  emit modified();
-  return true;
+Qt::ItemFlags
+GenericListWrapper::flags(const QModelIndex &index) const {
+  if (index.isValid())
+    return QAbstractListModel::flags(index) | Qt::ItemIsDragEnabled;
+  return QAbstractListModel::flags(index) | Qt::ItemIsDropEnabled;
+}
+
+Qt::DropActions
+GenericListWrapper::supportedDropActions() const {
+  return Qt::MoveAction;
 }
 
 bool
-GenericListWrapper::moveUp(int first, int last) {
-  if ((0>=first) || (last>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), first, last, QModelIndex(), first-1);
-  _list->moveUp(first, last);
-  endMoveRows();
-  emit modified();
-  return true;
-}
+GenericListWrapper::moveRows(const QModelIndex &sourceParent, int sourceRow, int count,
+                             const QModelIndex &destinationParent, int destinationChild)
+{
+  logDebug() << "Move " << count << " rows from " << sourceRow << " to " << destinationChild;
 
-bool
-GenericListWrapper::moveDown(int row) {
-  if ((0>row) || ((row+1)>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), row, row, QModelIndex(), row+2);
-  _list->moveDown(row);
+  beginMoveRows(sourceParent, sourceRow, sourceRow+count-1,
+                destinationParent, destinationChild);
+  bool success = _list->move(sourceRow, count, destinationChild);
   endMoveRows();
-  emit modified();
-  return true;
-}
 
-bool
-GenericListWrapper::moveDown(int first, int last) {
-  if ((0>first) || ((last+1)>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), first, last, QModelIndex(), last+2);
-  _list->moveDown(first, last);
-  endMoveRows();
-  emit modified();
-  return true;
+  return success;
 }
 
 
@@ -112,7 +95,7 @@ GenericListWrapper::onItemModified(int idx) {
  * Implementation of GenericTableWrapper
  * ********************************************************************************************* */
 GenericTableWrapper::GenericTableWrapper(AbstractConfigObjectList *list, QObject *parent)
-  : QAbstractTableModel(parent), _list(list)
+  : QAbstractTableModel(parent), _list(list), _insertRow(-1)
 {
   if (nullptr == _list)
     return;
@@ -131,48 +114,63 @@ GenericTableWrapper::rowCount(const QModelIndex &index) const {
   return _list->count();
 }
 
-bool
-GenericTableWrapper::moveUp(int row) {
-  if ((0>=row) || (row>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), row, row, QModelIndex(), row-1);
-  _list->moveUp(row);
-  endMoveRows();
-  emit modified();
-  return true;
+Qt::ItemFlags
+GenericTableWrapper::flags(const QModelIndex &index) const {
+  if (index.isValid())
+    return QAbstractTableModel::flags(index) | Qt::ItemIsDragEnabled;
+  return QAbstractTableModel::flags(index) | Qt::ItemIsDropEnabled;
+}
+
+Qt::DropActions
+GenericTableWrapper::supportedDropActions() const {
+  return Qt::MoveAction;
 }
 
 bool
-GenericTableWrapper::moveUp(int first, int last) {
-  if ((0>=first) || (last>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), first, last, QModelIndex(), first-1);
-  _list->moveUp(first, last);
-  endMoveRows();
-  emit modified();
-  return true;
+GenericTableWrapper::insertRows(int row, int count, const QModelIndex &parent) {
+  Q_UNUSED(parent); Q_UNUSED(count);
+
+  if (-1 == _insertRow) {
+    _insertRow = row;
+    return true;
+  }
+  return false;
 }
 
 bool
-GenericTableWrapper::moveDown(int row) {
-  if ((0>row) || ((row+1)>=_list->count()))
+GenericTableWrapper::removeRows(int row, int count, const QModelIndex &parent) {
+  Q_UNUSED(parent);
+
+  if (-1 == _insertRow)
     return false;
-  beginMoveRows(QModelIndex(), row, row, QModelIndex(), row+2);
-  _list->moveDown(row);
-  endMoveRows();
-  emit modified();
-  return true;
+
+  bool success = moveRows(QModelIndex(), row, count,
+                          QModelIndex(), _insertRow);
+  _insertRow = -1;
+  return success;
 }
 
 bool
-GenericTableWrapper::moveDown(int first, int last) {
-  if ((0>first) || ((last+1)>=_list->count()))
-    return false;
-  beginMoveRows(QModelIndex(), first, last, QModelIndex(), last+2);
-  _list->moveDown(first, last);
+GenericTableWrapper::moveRows(const QModelIndex &sourceParent, int sourceRow, int count,
+                             const QModelIndex &destinationParent, int destinationChild)
+{
+  logDebug() << "Move " << count << " rows from " << sourceRow << " to " << destinationChild;
+
+  beginMoveRows(sourceParent, sourceRow, sourceRow+count-1,
+                destinationParent, destinationChild);
+  bool success = _list->move(sourceRow, count, destinationChild);
   endMoveRows();
-  emit modified();
-  return true;
+
+  return success;
+}
+
+bool
+GenericTableWrapper::canDropMimeData(const QMimeData *data, Qt::DropAction action,
+                                 int row, int column, const QModelIndex &parent) const
+{
+  if (0 != column)
+    return false;
+  return QAbstractTableModel::canDropMimeData(data, action, row, column, parent);
 }
 
 void
@@ -213,12 +211,7 @@ ChannelListWrapper::ChannelListWrapper(ChannelList *list, QObject *parent)
 int
 ChannelListWrapper::columnCount(const QModelIndex &index) const {
   Q_UNUSED(index);
-  return 20;
-}
-
-inline QString formatFrequency(float f) {
-  int val = std::round(f*10000);
-  return QString::number(double(val)/10000, 'f', 4);
+  return 21;
 }
 
 QVariant
@@ -235,15 +228,15 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
     QColor inactive = palette.color(QPalette::Inactive, QPalette::Text);
     bool isDigital = dynamic_cast<ChannelList *>(_list)->channel(index.row())->is<DMRChannel>();
     switch(index.column()) {
-    case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8:
+    case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
       return active;
-    case 9: case 10: case 11: case 12: case 13:
+    case 11: case 12: case 13: case 14:
       return (isDigital ? active : inactive);
-    case 14:
-      return active;
     case 15:
+      return active;
+    case 16:
       return (isDigital ? active : inactive);
-    case 16: case 17: case 18: case 19:
+    case 17: case 18: case 19: case 20:
       return (isDigital ? inactive : active);
     }
   }
@@ -256,18 +249,15 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
   switch (index.column()) {
   case 0:
     if (channel->is<FMChannel>())
-      return tr("Analog");
+      return tr("FM");
     else
-      return tr("Digital");
+      return tr("DMR");
   case 1:
     return channel->name();
   case 2:
-    return formatFrequency(channel->rxFrequency());
+    return channel->rxFrequency().format(Frequency::Format::MHz);
   case 3:
-    if (channel->txFrequency()<channel->rxFrequency())
-      return formatFrequency(channel->txFrequency()-channel->rxFrequency());
-    else
-      return formatFrequency(channel->txFrequency());
+    return channel->txFrequency().format(Frequency::Format::MHz);
   case 4:
     if (channel->defaultPower())
       return tr("[Default]");
@@ -306,44 +296,53 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
     if (channel->scanList()) {
       return channel->scanList()->name();
     } else {
-      return tr("-");
+      return QString("-");
     }
-  case 9:
+  case 9: { // Collect zones, the channel is a member of
+      QStringList zones;
+      for(int i=0;i<channel->config()->zones()->count(); i++) {
+        Zone *zone = channel->config()->zones()->zone(i);
+        if (zone->contains(channel))
+          zones.append(zone->name());
+      }
+      return zones.join(", ");
+    } break;
+  case 10:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       return digi->colorCode();
     } else if (channel->is<FMChannel>()) {
       return tr("[None]");
     }
     break;
-  case 10:
+  case 11:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       return (DMRChannel::TimeSlot::TS1 == digi->timeSlot()) ? 1 : 2;
     } else if (channel->is<FMChannel>()) {
       return tr("[None]");
     }
     break;
-  case 11:
+  case 12:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       if (digi->groupListObj()) {
         return digi->groupListObj()->name();
       } else {
-        return tr("-");
+        return QString("-");
       }
     } else if (channel->is<FMChannel>()) {
       return tr("[None]");
     }
     break;
-  case 12:
+  case 13:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       if (digi->txContactObj())
         return digi->txContactObj()->name();
       else
-        return tr("-");
+        return QString("-");
     } else if (channel->is<FMChannel>()) {
       return tr("[None]");
     }
     break;
-  case 13:
+  case 14:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       if ((nullptr == digi->radioIdObj()) || (DefaultRadioID::get() == digi->radioIdObj()))
         return tr("[Default]");
@@ -352,23 +351,23 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
       return tr("[None]");
     }
     break;
-  case 14:
+  case 15:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       if (digi->aprsObj())
         return digi->aprsObj()->name();
       else
-        return tr("-");
+        return QString("-");
     } else if (FMChannel *analog = channel->as<FMChannel>()) {
       if (analog->aprsSystem())
         return analog->aprsSystem()->name();
       else
-        return tr("-");
+        return QString("-");
     }
     break;
-  case 15:
+  case 16:
     if (DMRChannel *digi = channel->as<DMRChannel>()) {
       if (nullptr == digi->roamingZone())
-        return tr("-");
+        return QString("-");
       else if (DefaultRoamingZone::get() == digi->roamingZone())
         return tr("[Default]");
       return digi->roamingZone()->name();
@@ -376,7 +375,7 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
       return tr("[None]");
     }
     break;
-  case 16:
+  case 17:
     if (channel->is<DMRChannel>()) {
       return tr("[None]");
     } else if (FMChannel *analog = channel->as<FMChannel>()) {
@@ -388,7 +387,7 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
         return analog->squelch();
     }
     break;
-  case 17:
+  case 18:
     if (channel->is<DMRChannel>()) {
       return tr("[None]");
     } else if (FMChannel *analog = channel->as<FMChannel>()) {
@@ -398,7 +397,7 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
         return Signaling::codeLabel(analog->rxTone());
     }
     break;
-  case 18:
+  case 19:
     if (channel->is<DMRChannel>()) {
       return tr("[None]");
     } else if (FMChannel *analog = channel->as<FMChannel>()) {
@@ -408,7 +407,7 @@ ChannelListWrapper::data(const QModelIndex &index, int role) const {
         return Signaling::codeLabel(analog->txTone());
     }
     break;
-  case 19:
+  case 20:
     if (channel->is<DMRChannel>()) {
       return tr("[None]");
     } else if (FMChannel *analog = channel->as<FMChannel>()) {
@@ -440,17 +439,18 @@ ChannelListWrapper::headerData(int section, Qt::Orientation orientation, int rol
   case 6: return tr("Rx Only");
   case 7: return tr("Admit");
   case 8: return tr("Scanlist");
-  case 9: return tr("CC");
-  case 10: return tr("TS");
-  case 11: return tr("RX Group List");
-  case 12: return tr("TX Contact");
-  case 13: return tr("DMR ID");
-  case 14: return tr("GPS/APRS");
-  case 15: return tr("Roaming");
-  case 16: return tr("Squelch");
-  case 17: return tr("Rx Tone");
-  case 18: return tr("Tx Tone");
-  case 19: return tr("Bandwidth");
+  case 9: return tr("Zones");
+  case 10: return tr("CC");
+  case 11: return tr("TS");
+  case 12: return tr("RX Group List");
+  case 13: return tr("TX Contact");
+  case 14: return tr("DMR ID");
+  case 15: return tr("GPS/APRS");
+  case 16: return tr("Roaming");
+  case 17: return tr("Squelch");
+  case 18: return tr("Rx Tone");
+  case 19: return tr("Tx Tone");
+  case 20: return tr("Bandwidth");
     default:
       break;
   }
@@ -494,7 +494,7 @@ RoamingChannelListWrapper::RoamingChannelListWrapper(RoamingChannelList *list, Q
 int
 RoamingChannelListWrapper::columnCount(const QModelIndex &index) const {
   Q_UNUSED(index);
-  return 5;
+  return 6;
 }
 
 QVariant
@@ -507,11 +507,8 @@ RoamingChannelListWrapper::data(const QModelIndex &index, int role) const {
   // Dispatch by column
   switch (index.column()) {
   case 0: return ch->name();
-  case 1: return ch->rxFrequency();
-  case 2:
-    if (ch->rxFrequency() == ch->txFrequency())
-      return ch->txFrequency();
-    return ch->txFrequency()-ch->rxFrequency();
+  case 1: return ch->rxFrequency().format(Frequency::Format::MHz);
+  case 2: return ch->txFrequency().format(Frequency::Format::MHz);
   case 3:
     if (ch->colorCodeOverridden())
       return ch->colorCode();
@@ -524,6 +521,16 @@ RoamingChannelListWrapper::data(const QModelIndex &index, int role) const {
       }
     }
     return tr("[Selected]");
+  case 5: {
+      QStringList zones;
+      for(int i=0;i<ch->config()->roamingZones()->count(); i++) {
+        RoamingZone *zone = ch->config()->roamingZones()->zone(i);
+        if (zone->contains(ch))
+          zones.append(zone->name());
+      }
+      return zones.join(", ");
+    } break;
+
   default: break;
   }
 
@@ -538,8 +545,9 @@ RoamingChannelListWrapper::headerData(int section, Qt::Orientation orientation,
   case 0: return tr("Name");
   case 1: return tr("RX Frequency");
   case 2: return tr("TX Frequency");
-  case 3: return tr("TS");
-  case 4: return tr("CC");
+  case 3: return tr("CC");
+  case 4: return tr("TS");
+  case 5: return tr("Zones");
   default: break;
   }
   return QVariant();
@@ -705,7 +713,7 @@ PositioningSystemListWrapper::data(const QModelIndex &index, int role) const {
     else if (sys->is<APRSSystem>())
       return tr("APRS");
     else
-      return tr("OOps!");
+      return QString("Oops!");
   case 1:
     return sys->name();
   case 2:
@@ -714,7 +722,7 @@ PositioningSystemListWrapper::data(const QModelIndex &index, int role) const {
         return tr("[None]");
       return sys->as<GPSSystem>()->contactObj()->name();
     } else if (sys->is<APRSSystem>())
-      return tr("%1-%2").arg(sys->as<APRSSystem>()->destination())
+      return QString("%1-%2").arg(sys->as<APRSSystem>()->destination())
           .arg(sys->as<APRSSystem>()->destSSID());
     break;
   case 3:
@@ -726,7 +734,7 @@ PositioningSystemListWrapper::data(const QModelIndex &index, int role) const {
       return sys->as<GPSSystem>()->revertChannel()->name();
     } else if (sys->is<APRSSystem>())
       return ((nullptr != sys->as<APRSSystem>()->revertChannel()) ?
-                sys->as<APRSSystem>()->revertChannel()->name() : tr("OOPS!"));
+                sys->as<APRSSystem>()->revertChannel()->name() : QString("Oops!"));
     break;
   case 5:
     if (sys->is<GPSSystem>())
