@@ -108,8 +108,8 @@ DigitalChannelDialog::construct() {
   voxDefault->setChecked(true); voxValue->setValue(0); voxValue->setEnabled(false);
 
   channelName->setText(_myChannel->name());
-  rxFrequency->setText(format_frequency(_myChannel->rxFrequency()));
-  txFrequency->setText(format_frequency(_myChannel->txFrequency()));
+  rxFrequency->setText(_myChannel->rxFrequency().format(Frequency::Format::MHz));
+  txFrequency->setText(_myChannel->txFrequency().format(Frequency::Format::MHz));
   if (! _myChannel->defaultPower()) {
     powerDefault->setChecked(false); powerValue->setEnabled(true);
     switch (_myChannel->power()) {
@@ -157,8 +157,8 @@ DigitalChannelDialog::channel()
 {
   _myChannel->setRadioIdObj(dmrID->currentData().value<DMRRadioID*>());
   _myChannel->setName(channelName->text());
-  _myChannel->setRXFrequency(rxFrequency->text().toDouble());
-  _myChannel->setTXFrequency(txFrequency->text().toDouble());
+  _myChannel->setRXFrequency(Frequency::fromString(rxFrequency->text()));
+  _myChannel->setTXFrequency(Frequency::fromString(txFrequency->text()));
   if (powerDefault->isChecked())
     _myChannel->setDefaultPower();
   else
