@@ -7,25 +7,11 @@
 #include <QTest>
 
 DMR6X2UVTest::DMR6X2UVTest(QObject *parent)
-  : QObject(parent)
+  : UnitTestBase(parent)
 {
   // pass...
 }
 
-void
-DMR6X2UVTest::initTestCase() {
-  ErrorStack err;
-  if (! _basicConfig.readYAML(":/data/config_test.yaml", err)) {
-    QFAIL(QString("Cannot open codeplug file: %1")
-          .arg(err.format()).toStdString().c_str());
-  }
-}
-
-void
-DMR6X2UVTest::cleanupTestCase() {
-  // clear codeplug
-  _basicConfig.clear();
-}
 
 void
 DMR6X2UVTest::testBasicConfigEncoding() {
@@ -33,7 +19,7 @@ DMR6X2UVTest::testBasicConfigEncoding() {
   Codeplug::Flags flags; flags.updateCodePlug=false;
   DMR6X2UVCodeplug codeplug;
   if (! codeplug.encode(&_basicConfig, flags, err)) {
-    QFAIL(QString("Cannot encode codeplug for BTECH DMR-6X2UV: {}")
+    QFAIL(QString("Cannot encode codeplug for BTECH DMR-6X2UV: %1")
           .arg(err.format()).toStdString().c_str());
   }
 }
@@ -44,15 +30,38 @@ DMR6X2UVTest::testBasicConfigDecoding() {
   Codeplug::Flags flags; flags.updateCodePlug=false;
   DMR6X2UVCodeplug codeplug;
   if (! codeplug.encode(&_basicConfig, flags, err)) {
+    QFAIL(QString("Cannot encode codeplug for BTECH DMR-6X2UV: %1")
+          .arg(err.format()).toStdString().c_str());
+  }
+
+  Config config;
+  if (! codeplug.decode(&config, err)) {
+    QFAIL(QString("Cannot decode codeplug for BETCH DMR-6X2UV: %1")
+          .arg(err.format()).toStdString().c_str());
+  }
+}
+
+void
+DMR6X2UVTest::testChannelFrequency() {
+  ErrorStack err;
+  Codeplug::Flags flags; flags.updateCodePlug=false;
+  D868UVCodeplug codeplug;
+  codeplug.clear();
+  if (! codeplug.encode(&_channelFrequencyConfig, flags, err)) {
     QFAIL(QString("Cannot encode codeplug for BTECH DMR-6X2UV: {}")
           .arg(err.format()).toStdString().c_str());
   }
 
   Config config;
   if (! codeplug.decode(&config, err)) {
-    QFAIL(QString("Cannot decode codeplug for BETCH DMR-6X2UV: {}")
+    QFAIL(QString("Cannot decode codeplug for BTECH DMR-6X2UV: {}")
           .arg(err.format()).toStdString().c_str());
   }
+
+  QCOMPARE(config.channelList()->channel(0)->rxFrequency(),
+           Frequency::fromHz(123456780ULL));
+  QCOMPARE(config.channelList()->channel(0)->txFrequency(),
+           Frequency::fromHz(999999990ULL));
 }
 
 QTEST_GUILESS_MAIN(DMR6X2UVTest)
